@@ -7,6 +7,8 @@ Copyrighted by Massimo Di Pierro <mdipierro@cs.depaul.edu>
 License: LGPLv3 (http://www.gnu.org/licenses/lgpl.html)
 """
 
+import re
+
 __all__ = ['HTTP', 'redirect']
 
 defined_status = {
@@ -40,6 +42,7 @@ defined_status = {
     415: 'UNSUPPORTED MEDIA TYPE',
     416: 'REQUESTED RANGE NOT SATISFIABLE',
     417: 'EXPECTATION FAILED',
+    422: 'UNPROCESSABLE ENTITY',
     500: 'INTERNAL SERVER ERROR',
     501: 'NOT IMPLEMENTED',
     502: 'BAD GATEWAY',
@@ -56,6 +59,7 @@ try:
 except NameError:
     BaseException = Exception
 
+regex_status = re.compile('^\d{3} \w+$')
 
 class HTTP(BaseException):
 
@@ -63,48 +67,64 @@ class HTTP(BaseException):
         self,
         status,
         body='',
+        cookies=None,
         **headers
         ):
         self.status = status
         self.body = body
         self.headers = headers
+        self.cookies2headers(cookies)
 
-    def to(self, responder):
-        if self.status in defined_status:
-            status = '%d %s' % (self.status, defined_status[self.status])
+    def cookies2headers(self,cookies):
+        if cookies and len(cookies)>0:
+            self.headers['Set-Cookie'] = [
+                str(cookie)[11:] for cookie in cookies.values()]
+
+    def to(self, responder, env=None):
+        env = env or {}
+        status = self.status
+        headers = self.headers
+        if status in defined_status:
+            status = '%d %s' % (status, defined_status[status])
         else:
-            status = str(self.status) + ' '
-        if not 'Content-Type' in self.headers:
-            self.headers['Content-Type'] = 'text/html; charset=UTF-8'
+            status = str(status)
+            if not regex_status.match(status):
+                status = '500 %s' % (defined_status[500])
+        headers.setdefault('Content-Type','text/html; charset=UTF-8')
         body = self.body
         if status[:1] == '4':
             if not body:
                 body = status
             if isinstance(body, str):
-                if len(body)<512 and self.headers['Content-Type'].startswith('text/html'):
+                if len(body)<512 and \
+                        headers['Content-Type'].startswith('text/html'):
                     body += '<!-- %s //-->' % ('x'*512) ### trick IE
-                self.headers['Content-Length'] = len(body)
-        headers = []
-        for (k, v) in self.headers.items():
+                headers['Content-Length'] = len(body)
+        rheaders = []
+        for k, v in headers.iteritems():
             if isinstance(v, list):
-                for item in v:
-                    headers.append((k, str(item)))
-            else:
-                headers.append((k, str(v)))
-        responder(status, headers)
-        if hasattr(body, '__iter__') and not isinstance(self.body, str):
+                rheaders += [(k, str(item)) for item in v]
+            elif not v is None:
+                rheaders.append((k, str(v)))
+        responder(status, rheaders)
+        if env.get('request_method','')=='HEAD':
+            return ['']
+        elif isinstance(body,str):
+            return [body]
+        elif hasattr(body, '__iter__'):
             return body
-        return [str(body)]
+        else:
+            return [str(body)]
 
     @property
     def message(self):
-        '''
+        """
         compose a message describing this exception
 
-        "status defined_status [web2py_error]"
+            "status defined_status [web2py_error]"
 
         message elements that are not defined are omitted
-        '''
+        """
         msg = '%(status)d'
         if self.status in defined_status:
             msg = '%(status)d %(defined_status)s'
@@ -119,13 +139,19 @@ class HTTP(BaseException):
         return self.message
 
 
-def redirect(location, how=303):
-    if not location:
-        return
-    location = location.replace('\r', '%0D').replace('\n', '%0A')
-    raise HTTP(how,
-               'You are being redirected <a href="%s">here</a>' % location,
-               Location=location)
+def redirect(location, how=303, client_side=False):
+    if location:
+        from gluon import current
+        loc = location.replace('\r', '%0D').replace('\n', '%0A')
+        if client_side and current.request.ajax:
+            raise HTTP(200, **{'web2py-redirect-location': loc})
+        else:
+            raise HTTP(how,
+                       'You are being redirected <a href="%s">here</a>' % loc,
+                       Location=loc)
+
+
+
 
 
 
