@@ -21,20 +21,20 @@ import re
 import copy
 import sys
 import time
-import thread
 import datetime
 import signal
 import socket
 import tempfile
 import random
 import string
-from fileutils import abspath, write_file, parse_version
+import urllib2
+from thread import allocate_lock
+
+from fileutils import abspath, write_file, parse_version, copystream
 from settings import global_settings
 from admin import add_path_first, create_missing_folders, create_missing_app_folders
 from globals import current
 
-from custom_import import custom_import_install
-
 #  Remarks:
 #  calling script has inserted path to script directory into sys.path
 #  applications_parent (path to applications/, site-packages/ etc)
@@ -51,25 +51,25 @@ from custom_import import custom_import_install
 #  The two are identical unless web2py_path is changed via the web2py.py -f folder option
 #  main.web2py_path is the same as applications_parent (for backward compatibility)
 
-if not hasattr(os, 'mkdir'):
-    global_settings.db_sessions = True
-if global_settings.db_sessions is not True:
-    global_settings.db_sessions = set()
-global_settings.gluon_parent = os.environ.get('web2py_path', os.getcwd())
-global_settings.applications_parent = global_settings.gluon_parent
 web2py_path = global_settings.applications_parent # backward compatibility
-global_settings.app_folders = set()
-global_settings.debugging = False
-
-custom_import_install(web2py_path)
 
 create_missing_folders()
 
 # set up logging for subsequent imports
 import logging
 import logging.config
+
+# This needed to prevent exception on Python 2.5:
+# NameError: name 'gluon' is not defined
+# See http://bugs.python.org/issue1436
+import gluon.messageboxhandler
+logging.gluon = gluon
+
+exists = os.path.exists
+pjoin = os.path.join
+
 logpath = abspath("logging.conf")
-if os.path.exists(logpath):
+if exists(logpath):
     logging.config.fileConfig(abspath("logging.conf"))
 else:
     logging.basicConfig()
@@ -80,15 +80,16 @@ from http import HTTP, redirect
 from globals import Request, Response, Session
 from compileapp import build_environment, run_models_in, \
     run_controller_in, run_view_in
-from fileutils import copystream
 from contenttype import contenttype
 from dal import BaseAdapter
 from settings import global_settings
 from validators import CRYPT
-from cache import Cache
-from html import URL as Url
+from cache import CacheInRam
+from html import URL, xmlescape
+from utils import is_valid_ip_address
+from rewrite import load, url_in, THREAD_LOCAL as rwthread, \
+    try_rewrite_on_error, fixup_missing_path_info
 import newcron
-import rewrite
 
 __all__ = ['wsgibase', 'save_password', 'appfactory', 'HttpServer']
 
@@ -100,10 +101,15 @@ requests = 0    # gc timer
 # pattern used to validate client address
 regex_client = re.compile('[\w\-:]+(\.[\w\-]+)*\.?')  # ## to account for IPV6
 
-version_info = open(abspath('VERSION', gluon=True), 'r')
-web2py_version = parse_version(version_info.read().strip())
-version_info.close()
-global_settings.web2py_version = web2py_version
+try:
+    version_info = open(pjoin(global_settings.gluon_parent, 'VERSION'), 'r')
+    raw_version_string = version_info.read().strip()
+    version_info.close()
+    global_settings.web2py_version = parse_version(raw_version_string)
+except:
+    raise RuntimeError, "Cannot determine web2py version"
+
+web2py_version = global_settings.web2py_version
 
 try:
     import rocket
@@ -111,7 +117,9 @@ except:
     if not global_settings.web2py_runtime_gae:
         logger.warn('unable to import Rocket')
 
-rewrite.load()
+load()
+
+HTTPS_SCHEMES = set(('https','HTTPS'))
 
 def get_client(env):
     """
@@ -122,37 +130,46 @@ def get_client(env):
     """
     g = regex_client.search(env.get('http_x_forwarded_for', ''))
     if g:
-        return g.group()
-    g = regex_client.search(env.get('remote_addr', ''))
-    if g:
-        return g.group()
-    return '127.0.0.1'
+        client = (g.group() or '').split(',')[0]
+    else:
+        g = regex_client.search(env.get('remote_addr', ''))
+        if g:
+            client = g.group()
+        else:
+            client = '127.0.0.1'
+    if not is_valid_ip_address(client):
+        raise HTTP(400,"Bad Request (request.client=%s)" % client)
+    return client
 
 def copystream_progress(request, chunk_size= 10**5):
     """
     copies request.env.wsgi_input into request.body
-    and stores progress upload status in cache.ram
+    and stores progress upload status in cache_ram
     X-Progress-ID:length and X-Progress-ID:uploaded
     """
-    if not request.env.content_length:
+    env = request.env
+    if not env.content_length:
         return cStringIO.StringIO()
-    source = request.env.wsgi_input
-    size = int(request.env.content_length)
+    source = env.wsgi_input
+    try:
+        size = int(env.content_length)
+    except ValueError:
+        raise HTTP(400,"Invalid Content-Length header")
     dest = tempfile.TemporaryFile()
     if not 'X-Progress-ID' in request.vars:
         copystream(source, dest, size, chunk_size)
         return dest
     cache_key = 'X-Progress-ID:'+request.vars['X-Progress-ID']
-    cache = Cache(request)
-    cache.ram(cache_key+':length', lambda: size, 0)
-    cache.ram(cache_key+':uploaded', lambda: 0, 0)
+    cache_ram = CacheInRam(request)  # same as cache.ram because meta_storage
+    cache_ram(cache_key+':length', lambda: size, 0)
+    cache_ram(cache_key+':uploaded', lambda: 0, 0)
     while size > 0:
         if size < chunk_size:
             data = source.read(size)
-            cache.ram.increment(cache_key+':uploaded', size)
+            cache_ram.increment(cache_key+':uploaded', size)
         else:
             data = source.read(chunk_size)
-            cache.ram.increment(cache_key+':uploaded', chunk_size)
+            cache_ram.increment(cache_key+':uploaded', chunk_size)
         length = len(data)
         if length > size:
             (data, length) = (data[:size], size)
@@ -163,8 +180,8 @@ def copystream_progress(request, chunk_size= 10**5):
         if length < chunk_size:
             break
     dest.seek(0)
-    cache.ram(cache_key+':length', None)
-    cache.ram(cache_key+':uploaded', None)
+    cache_ram(cache_key+':length', None)
+    cache_ram(cache_key+':uploaded', None)
     return dest
 
 
@@ -202,8 +219,7 @@ def serve_controller(request, response, session):
     page = run_controller_in(request.controller, request.function, environment)
     if isinstance(page, dict):
         response._vars = page
-        for key in page:
-            response._view_environment[key] = page[key]
+        response._view_environment.update(page)
         run_view_in(response._view_environment)
         page = response.body.getvalue()
     # logic to garbage collect after exec, not always, once every 100 requests
@@ -211,6 +227,20 @@ def serve_controller(request, response, session):
     requests = ('requests' in globals()) and (requests+1) % 100 or 0
     if not requests: gc.collect()
     # end garbage collection logic
+
+    # ##################################################
+    # set default headers it not set
+    # ##################################################
+
+    default_headers = [
+        ('Content-Type', contenttype('.'+request.extension)),
+        ('Cache-Control','no-store, no-cache, must-revalidate, post-check=0, pre-check=0'),
+        ('Expires', time.strftime('%a, %d %b %Y %H:%M:%S GMT', 
+                                  time.gmtime())),
+        ('Pragma', 'no-cache')]
+    for key,value in default_headers:
+        response.headers.setdefault(key,value)
+
     raise HTTP(response.status, page, **response.headers)
 
 
@@ -247,7 +277,8 @@ def middleware_aux(request, response, *middleware_apps):
         for item in middleware_apps:
             app=item(app)
         def caller(app):
-            return app(request.wsgi.environ,request.wsgi.start_response)
+            wsgi = request.wsgi
+            return app(wsgi.environ, wsgi.start_response)
         return lambda caller=caller, app=app: caller(app)
     return middleware
 
@@ -260,7 +291,8 @@ def environ_aux(environ,request):
 def parse_get_post_vars(request, environ):
 
     # always parse variables in URL for GET, POST, PUT, DELETE, etc. in get_vars
-    dget = cgi.parse_qsl(request.env.query_string or '', keep_blank_values=1)
+    env = request.env
+    dget = cgi.parse_qsl(env.query_string or '', keep_blank_values=1)
     for (key, value) in dget:
         if key in request.get_vars:
             if isinstance(request.get_vars[key], list):
@@ -272,12 +304,15 @@ def parse_get_post_vars(request, environ):
         request.vars[key] = request.get_vars[key]
 
     # parse POST variables on POST, PUT, BOTH only in post_vars
-    request.body = copystream_progress(request) ### stores request body
-    if (request.body and request.env.request_method in ('POST', 'PUT', 'BOTH')):
-        dpost = cgi.FieldStorage(fp=request.body,environ=environ,keep_blank_values=1)
+    try:
+        request.body = body = copystream_progress(request)
+    except IOError:
+        raise HTTP(400,"Bad Request - HTTP body is incomplete")
+    if (body and env.request_method in ('POST', 'PUT', 'BOTH')):
+        dpost = cgi.FieldStorage(fp=body,environ=environ,keep_blank_values=1)
         # The same detection used by FieldStorage to detect multipart POSTs
         is_multipart = dpost.type[:10] == 'multipart/'
-        request.body.seek(0)
+        body.seek(0)
         isle25 = sys.version_info[1] <= 5
 
         def listify(a):
@@ -287,13 +322,16 @@ def parse_get_post_vars(request, environ):
         except TypeError:
             keys = []
         for key in keys:
+            if key is None: continue # not sure why cgi.FieldStorage returns None key
             dpk = dpost[key]
             # if en element is not a file replace it with its value else leave it alone
             if isinstance(dpk, list):
-                if not dpk[0].filename:
-                    value = [x.value for x in dpk]
-                else:
-                    value = [x for x in dpk]
+                value = []
+                for _dpk in dpk:
+                    if not _dpk.filename:
+                        value.append(_dpk.value)
+                    else:
+                        value.append(_dpk)
             elif not dpk.filename:
                 value = dpk.value
             else:
@@ -347,9 +385,10 @@ def wsgibase(environ, responder):
     request = Request()
     response = Response()
     session = Session()
-    request.env.web2py_path = global_settings.applications_parent
-    request.env.web2py_version = web2py_version
-    request.env.update(global_settings)
+    env = request.env
+    env.web2py_path = global_settings.applications_parent
+    env.web2py_version = web2py_version
+    env.update(global_settings)
     static_file = False
     try:
         try:
@@ -363,76 +402,78 @@ def wsgibase(environ, responder):
                 # serve file if static
                 # ##################################################
 
-                if not environ.get('PATH_INFO',None) and \
-                        environ.get('REQUEST_URI',None):
-                    # for fcgi, get path_info and query_string from request_uri
-                    items = environ['REQUEST_URI'].split('?')
-                    environ['PATH_INFO'] = items[0]
-                    if len(items) > 1:
-                        environ['QUERY_STRING'] = items[1]
-                    else:
-                        environ['QUERY_STRING'] = ''
-                if not environ.get('HTTP_HOST',None):
-                    environ['HTTP_HOST'] = '%s:%s' % (environ.get('SERVER_NAME'),
-                                                      environ.get('SERVER_PORT'))
+                fixup_missing_path_info(environ)
+                (static_file, version, environ) = url_in(request, environ)
+                response.status = env.web2py_status_code or response.status
 
-                (static_file, environ) = rewrite.url_in(request, environ)
                 if static_file:
-                    if environ.get('QUERY_STRING', '')[:10] == 'attachment':
-                        response.headers['Content-Disposition'] = 'attachment'
+                    if environ.get('QUERY_STRING','').startswith(
+                        'attachment'):
+                        response.headers['Content-Disposition'] \
+                            = 'attachment'
+                    if version:
+                        response.headers['Cache-Control'] = 'max-age=315360000'
+                        response.headers['Expires'] = 'Thu, 31 Dec 2037 23:59:59 GMT'
                     response.stream(static_file, request=request)
 
                 # ##################################################
                 # fill in request items
                 # ##################################################
+                app = request.application ## must go after url_in!
 
-                http_host = request.env.http_host.split(':',1)[0]
-
-                local_hosts = [http_host,'::1','127.0.0.1','::ffff:127.0.0.1']
-                if not global_settings.web2py_runtime_gae:
-                    local_hosts += [socket.gethostname(),
-                                    socket.gethostbyname(http_host)]
-                request.client = get_client(request.env)
-                request.folder = abspath('applications',
-                                         request.application) + os.sep
-                x_req_with = str(request.env.http_x_requested_with).lower()
-                request.ajax = x_req_with == 'xmlhttprequest'
-                request.cid = request.env.http_web2py_component_element
-                request.is_local = request.env.remote_addr in local_hosts
-                request.is_https = request.env.wsgi_url_scheme \
-                    in ['https', 'HTTPS'] or request.env.https == 'on'
-
-                # ##################################################
-                # compute a request.uuid to be used for tickets and toolbar
-                # ##################################################
-
-                response.uuid = request.compute_uuid()
+                if not global_settings.local_hosts:
+                    local_hosts = ['127.0.0.1','::ffff:127.0.0.1']
+                    if not global_settings.web2py_runtime_gae:
+                        try:
+                            local_hosts.append(socket.gethostname())
+                        except TypeError:
+                            pass
+                        try:
+                            if env.server_name:
+                                local_hosts += [
+                                    env.server_name,
+                                    socket.gethostbyname(env.server_name)]
+                        except (socket.gaierror,TypeError):
+                            pass
+                    global_settings.local_hosts = local_hosts
+                else:
+                    local_hosts = global_settings.local_hosts
+                client = get_client(env)
+                x_req_with = str(env.http_x_requested_with).lower()
+                
+                request.update(
+                    client = client,
+                    folder = abspath('applications',app) + os.sep,
+                    ajax = x_req_with == 'xmlhttprequest',
+                    cid = env.http_web2py_component_element,
+                    is_local = env.remote_addr in local_hosts,
+                    is_https = env.wsgi_url_scheme in HTTPS_SCHEMES \
+                        or request.env.http_x_forwarded_proto in HTTPS_SCHEMES \
+                        or env.https=='on')
+                request.uuid = request.compute_uuid() # requires client
+                request.url = environ['PATH_INFO']
 
                 # ##################################################
                 # access the requested application
                 # ##################################################
 
-                if not os.path.exists(request.folder):
-                    if request.application == \
-                            rewrite.thread.routes.default_application \
-                            and request.application != 'welcome':
-                        request.application = 'welcome'
-                        redirect(Url(r=request))
-                    elif rewrite.thread.routes.error_handler:
-                        _handler = rewrite.thread.routes.error_handler
-                        redirect(Url(_handler['application'],
+                if not exists(request.folder):
+                    if app == rwthread.routes.default_application \
+                            and app != 'welcome':
+                        redirect(URL('welcome','default','index'))
+                    elif rwthread.routes.error_handler:
+                        _handler = rwthread.routes.error_handler
+                        redirect(URL(_handler['application'],
                                      _handler['controller'],
                                      _handler['function'],
-                                     args=request.application))
+                                     args=app))
                     else:
-                        raise HTTP(404, rewrite.thread.routes.error_message \
+                        raise HTTP(404, rwthread.routes.error_message \
                                        % 'invalid request',
                                    web2py_error='invalid application')
                 elif not request.is_local and \
-                        os.path.exists(os.path.join(request.folder,'DISABLED')):
-                    raise HTTP(200, "<html><body><h1>Down for maintenance</h1></body></html>")
-                request.url = Url(r=request, args=request.args,
-                                       extension=request.raw_extension)
+                        exists(pjoin(request.folder,'DISABLED')):
+                    raise HTTP(503, "<html><body><h1>Temporarily down for maintenance</h1></body></html>")
 
                 # ##################################################
                 # build missing folders
@@ -462,9 +503,9 @@ def wsgibase(environ, responder):
                 # load cookies
                 # ##################################################
 
-                if request.env.http_cookie:
+                if env.http_cookie:
                     try:
-                        request.cookies.load(request.env.http_cookie)
+                        request.cookies.load(env.http_cookie)
                     except Cookie.CookieError, e:
                         pass # invalid cookies
 
@@ -472,35 +513,25 @@ def wsgibase(environ, responder):
                 # try load session or create new session file
                 # ##################################################
 
-                session.connect(request, response)
-
-                # ##################################################
-                # set no-cache headers
-                # ##################################################
-
-                response.headers['Content-Type'] = \
-                    contenttype('.'+request.extension)
-                response.headers['Cache-Control'] = \
-                    'no-store, no-cache, must-revalidate, post-check=0, pre-check=0'
-                response.headers['Expires'] = \
-                    time.strftime('%a, %d %b %Y %H:%M:%S GMT', time.gmtime())
-                response.headers['Pragma'] = 'no-cache'
+                if not env.web2py_disable_session:
+                    session.connect(request, response)
 
                 # ##################################################
                 # run controller
                 # ##################################################
 
-                if global_settings.debugging and request.application != "admin":
+                if global_settings.debugging and app != "admin":
                     import gluon.debug
-                    # activate the debugger and wait to reach application code
+                    # activate the debugger
                     gluon.debug.dbg.do_debug(mainpyfile=request.folder)
-
+                
                 serve_controller(request, response, session)
 
             except HTTP, http_response:
-                if static_file:
-                    return http_response.to(responder)
 
+                if static_file:
+                    return http_response.to(responder,env=env)
+                        
                 if request.body:
                     request.body.close()
 
@@ -515,8 +546,10 @@ def wsgibase(environ, responder):
 
                 if response.do_not_commit is True:
                     BaseAdapter.close_all_instances(None)
-                elif response._custom_commit:
-                    response._custom_commit()
+                # elif response._custom_commit:
+                #     response._custom_commit()
+                elif response.custom_commit:
+                    BaseAdapter.close_all_instances(response.custom_commit)
                 else:
                     BaseAdapter.close_all_instances('commit')
 
@@ -524,29 +557,25 @@ def wsgibase(environ, responder):
                 # if session not in db try store session on filesystem
                 # this must be done after trying to commit database!
                 # ##################################################
-
+                    
                 session._try_store_on_disk(request, response)
+                
+                if request.cid:
+                    if response.flash:
+                        http_response.headers['web2py-component-flash'] = urllib2.quote(xmlescape(response.flash).replace('\n',''))
+                    if response.js:
+                        http_response.headers['web2py-component-command'] = response.js.replace('\n','')
 
                 # ##################################################
                 # store cookies in headers
                 # ##################################################
 
-                if request.cid:
-
-                    if response.flash and not 'web2py-component-flash' in http_response.headers:
-                        http_response.headers['web2py-component-flash'] = \
-                            str(response.flash).replace('\n','')
-                    if response.js and not 'web2py-component-command' in http_response.headers:
-                        http_response.headers['web2py-component-command'] = \
-                            response.js.replace('\n','')
-                if session._forget and \
-                        response.session_id_name in response.cookies:
-                    del response.cookies[response.session_id_name]
+                rcookies = response.cookies
+                if session._forget and response.session_id_name in rcookies:
+                    del rcookies[response.session_id_name]
                 elif session._secure:
-                    response.cookies[response.session_id_name]['secure'] = True
-                if len(response.cookies)>0:
-                    http_response.headers['Set-Cookie'] = \
-                        [str(cookie)[11:] for cookie in response.cookies.values()]
+                    rcookies[response.session_id_name]['secure'] = True
+                http_response.cookies2headers(rcookies)
                 ticket=None
 
             except RestrictedError, e:
@@ -565,7 +594,7 @@ def wsgibase(environ, responder):
                     BaseAdapter.close_all_instances('rollback')
 
                 http_response = \
-                    HTTP(500, rewrite.thread.routes.error_message_ticket % \
+                    HTTP(500, rwthread.routes.error_message_ticket % \
                              dict(ticket=ticket),
                          web2py_error='ticket %s' % ticket)
 
@@ -588,7 +617,7 @@ def wsgibase(environ, responder):
             e = RestrictedError('Framework', '', '', locals())
             ticket = e.log(request) or 'unrecoverable'
             http_response = \
-                HTTP(500, rewrite.thread.routes.error_message_ticket \
+                HTTP(500, rwthread.routes.error_message_ticket \
                          % dict(ticket=ticket),
                      web2py_error='ticket %s' % ticket)
 
@@ -598,13 +627,13 @@ def wsgibase(environ, responder):
             response.session_file.close()
 
     session._unlock(response)
-    http_response, new_environ = rewrite.try_rewrite_on_error(
+    http_response, new_environ = try_rewrite_on_error(
         http_response, request, environ, ticket)
     if not http_response:
         return wsgibase(new_environ,responder)
     if global_settings.web2py_crontype == 'soft':
         newcron.softcron(global_settings.applications_parent).start()
-    return http_response.to(responder)
+    return http_response.to(responder,env=env)
 
 
 def save_password(password, port):
@@ -623,7 +652,7 @@ def save_password(password, port):
         print '*********************************************************'
     elif password == '<recycle>':
         # reuse the current password if any
-        if os.path.exists(password_file):
+        if exists(password_file):
             return
         else:
             password = ''
@@ -654,9 +683,9 @@ def appfactory(wsgiapp=wsgibase,
             [, profilerfilename='profiler.log']]])
 
     """
-    if profilerfilename and os.path.exists(profilerfilename):
+    if profilerfilename and exists(profilerfilename):
         os.unlink(profilerfilename)
-    locker = thread.allocate_lock()
+    locker = allocate_lock()
 
     def app_with_logging(environ, responder):
         """
@@ -766,8 +795,7 @@ class HttpServer(object):
             global_settings.applications_parent = path
             os.chdir(path)
             [add_path_first(p) for p in (path, abspath('site-packages'), "")]
-            custom_import_install(web2py_path)
-            if os.path.exists("logging.conf"):
+            if exists("logging.conf"):
                 logging.config.fileConfig("logging.conf")
 
         save_password(password, port)
@@ -782,9 +810,9 @@ class HttpServer(object):
             logger.info('SSL is off')
         elif not rocket.ssl:
             logger.warning('Python "ssl" module unavailable. SSL is OFF')
-        elif not os.path.exists(ssl_certificate):
+        elif not exists(ssl_certificate):
             logger.warning('unable to open SSL certificate. SSL is OFF')
-        elif not os.path.exists(ssl_private_key):
+        elif not exists(ssl_private_key):
             logger.warning('unable to open SSL private key. SSL is OFF')
         else:
             sock_list.extend([ssl_private_key, ssl_certificate])
@@ -830,6 +858,3 @@ class HttpServer(object):
         except:
             pass
 
-
-
-
