@@ -13,6 +13,7 @@ if os.path.isdir('gluon'):
     sys.path.append(os.path.realpath('gluon')) # running from web2py base
 else:
     sys.path.append(os.path.realpath('../')) # running from gluon/tests/
+    os.environ['web2py_path'] = os.path.realpath('../../')  # for settings
 
 from rewrite import load, filter_url, filter_err, get_effective_router, map_url_out
 from html import URL
@@ -53,7 +54,7 @@ def setUpModule():
         routes = open(abspath('applications', 'examples', 'routes.py'), 'w')
         routes.write("routers=dict(examples=dict(default_function='exdef'))")
         routes.close()
-        
+
         #  create language files for examples app
         for lang in ('en', 'it'):
             os.mkdir(abspath('applications', 'examples', 'static', lang))
@@ -91,11 +92,11 @@ class TestRouter(unittest.TestCase):
         self.assertRaises(SyntaxError, load, rdict=dict(BASE=dict(), app=dict(default_application="name")))
         try:
             # 2.7+ only
-            self.assertRaisesRegexp(SyntaxError, "invalid syntax", 
+            self.assertRaisesRegexp(SyntaxError, "invalid syntax",
                 load, data='x:y')
-            self.assertRaisesRegexp(SyntaxError, "unknown key", 
+            self.assertRaisesRegexp(SyntaxError, "unknown key",
                 load, rdict=dict(BASE=dict(badkey="value")))
-            self.assertRaisesRegexp(SyntaxError, "BASE-only key", 
+            self.assertRaisesRegexp(SyntaxError, "BASE-only key",
                 load, rdict=dict(BASE=dict(), app=dict(default_application="name")))
         except AttributeError:
             pass
@@ -132,9 +133,9 @@ class TestRouter(unittest.TestCase):
         self.assertEqual(filter_url('http://domain.com/admin/default/abc', out=True), '/admin/abc')
 
     def test_router_specific(self):
-        """ 
-        Test app-specific routes.py 
-        
+        """
+        Test app-specific routes.py
+
         Note that make_apptree above created applications/examples/routes.py with a default_function.
         """
         load(rdict=dict())
@@ -163,7 +164,7 @@ class TestRouter(unittest.TestCase):
         self.assertEqual(filter_url('http://domain.com/welcome/default/index/arg1', out=True), '/index/arg1')
         self.assertEqual(filter_url('http://domain.com/welcome/default/abc', out=True), '/abc')
         self.assertEqual(filter_url('http://domain.com/welcome/default/admin', out=True), '/default/admin')
-        self.assertEqual(filter_url('http://domain.com/welcome/static/abc', out=True), 
+        self.assertEqual(filter_url('http://domain.com/welcome/static/abc', out=True),
             '/welcome/static/abc')
         self.assertEqual(filter_url('http://domain.com/welcome/appadmin/index', out=True), '/appadmin')
         self.assertEqual(filter_url('http://domain.com/welcome/appadmin/abc', out=True), '/appadmin/abc')
@@ -182,7 +183,7 @@ class TestRouter(unittest.TestCase):
         self.assertEqual(filter_url('http://domain.com/welcome/default/index', out=True), '/default')
         self.assertEqual(filter_url('http://domain.com/welcome/default/index/arg1', out=True), '/default/index/arg1')
         self.assertEqual(filter_url('http://domain.com/welcome/default/abc', out=True), '/default/abc')
-        self.assertEqual(filter_url('http://domain.com/welcome/static/abc', out=True), 
+        self.assertEqual(filter_url('http://domain.com/welcome/static/abc', out=True),
             '/welcome/static/abc')
         self.assertEqual(filter_url('http://domain.com/welcome/appadmin/index', out=True), '/appadmin')
         self.assertEqual(filter_url('http://domain.com/welcome/appadmin/abc', out=True), '/appadmin/abc')
@@ -345,8 +346,13 @@ class TestRouter(unittest.TestCase):
         self.assertEqual(filter_url('http://domain1.com/abc', app=True), 'app1')
         self.assertEqual(filter_url('http://www.domain1.com/abc', app=True), 'app1')
         self.assertEqual(filter_url('http://domain2.com/abc', app=True), 'app2')
+        self.assertEqual(filter_url('http://domain2.com/admin', app=True), 'admin')
+
+        routers['BASE']['exclusive_domain'] = True
+        load(rdict=routers)
         self.assertEqual(filter_url('http://domain2.com/admin', app=True), 'app2')
 
+
         self.assertEqual(filter_url('http://domain.com/goodapp', app=True), 'goodapp')
         self.assertRaises(HTTP, filter_url, 'http://domain.com/bad!app', app=True)
         try:
@@ -385,7 +391,7 @@ class TestRouter(unittest.TestCase):
         self.assertEqual(filter_url('http://domain2.com/f2'), '/welcome/default/f2')
         self.assertEqual(filter_url('http://domain2.com/other/f3'), '/welcome/other/f3')
 
-        
+
     def test_router_domains(self):
         '''
         Test URLs that map domains
@@ -427,8 +433,8 @@ class TestRouter(unittest.TestCase):
         self.assertEqual(filter_url('http://domain1.com/abc.html'), '/app1/c1/abc')
         self.assertEqual(filter_url('http://domain1.com/abc.css'), '/app1/c1/abc.css')
         self.assertEqual(filter_url('http://domain1.com/index/abc'), "/app1/c1/index ['abc']")
-        self.assertEqual(filter_url('http://domain2.com/app1'), "/app2a/c2a/app1")
-        
+        self.assertEqual(filter_url('http://domain2.com/app1'), "/app1/c1/f1")
+
         self.assertEqual(filter_url('https://domain1.com/app1/ctr/fcn', domain=('app1',None), out=True), "/ctr/fcn")
         self.assertEqual(filter_url('https://www.domain1.com/app1/ctr/fcn', domain=('app1',None), out=True), "/ctr/fcn")
 
@@ -474,6 +480,98 @@ class TestRouter(unittest.TestCase):
             pass
         self.assertEqual(filter_url('http://domain1.com/app1/c1/f1', domain=('app2b',None), host='domain2.com', out=True), "/app1")
 
+
+    def test_router_domains_ed(self):
+        '''
+        Test URLs that map domains with exclusive_domain set
+        '''
+        routers = dict(
+            BASE = dict(
+                applications = ['app1', 'app2', 'app2A', 'app3', 'app4', 'app5', 'app6'],
+                exclusive_domain = True,
+                domains = {
+                    #  two domains to the same app
+                    "domain1.com"     : "app1",
+                    "www.domain1.com" : "app1",
+                    #  same domain, two ports, to two apps
+                    "domain2.com"      : "app2a",
+                    "domain2.com:8080" : "app2b",
+                    #  two domains, same app, two controllers
+                    "domain3a.com" : "app3/c3a",
+                    "domain3b.com" : "app3/c3b",
+                    #  two domains, same app & controller, two functions
+                    "domain4a.com" : "app4/c4/f4a",
+                    "domain4b.com" : "app4/c4/f4b",
+                    #  http vs https
+                    "domain6.com:80"  : "app6",
+                    "domain6.com:443" : "app6s",
+                },
+            ),
+            app1 =  dict( default_controller = 'c1',  default_function = 'f1',  controllers = ['c1'], exclusive_domain=True, ),
+            app2a = dict( default_controller = 'c2a', default_function = 'f2a', controllers = ['c2a'], ),
+            app2b = dict( default_controller = 'c2b', default_function = 'f2b', controllers = ['c2b'], ),
+            app3 =  dict( controllers = ['c3a', 'c3b'], ),
+            app4 =  dict( default_controller = 'c4', controllers = ['c4']),
+            app5 =  dict( default_controller = 'c5', controllers = ['c5'], domain = 'localhost' ),
+            app6 =  dict( default_controller = 'c6',  default_function = 'f6',  controllers = ['c6'], ),
+            app6s =  dict( default_controller = 'c6s',  default_function = 'f6s',  controllers = ['c6s'], ),
+        )
+
+        load(rdict=routers)
+        self.assertEqual(filter_url('http://domain1.com/abc'), '/app1/c1/abc')
+        self.assertEqual(filter_url('http://domain1.com/c1/abc'), '/app1/c1/abc')
+        self.assertEqual(filter_url('http://domain1.com/abc.html'), '/app1/c1/abc')
+        self.assertEqual(filter_url('http://domain1.com/abc.css'), '/app1/c1/abc.css')
+        self.assertEqual(filter_url('http://domain1.com/index/abc'), "/app1/c1/index ['abc']")
+        self.assertEqual(filter_url('http://domain2.com/app1'), "/app2a/c2a/app1")
+
+        self.assertEqual(filter_url('https://domain1.com/app1/ctr/fcn', domain=('app1',None), out=True), "/ctr/fcn")
+        self.assertEqual(filter_url('https://www.domain1.com/app1/ctr/fcn', domain=('app1',None), out=True), "/ctr/fcn")
+
+        self.assertEqual(filter_url('http://domain2.com/abc'), '/app2a/c2a/abc')
+        self.assertEqual(filter_url('http://domain2.com:8080/abc'), '/app2b/c2b/abc')
+
+        self.assertEqual(filter_url('http://domain2.com/app2a/ctr/fcn', domain=('app2a',None), out=True), "/ctr/fcn")
+        self.assertEqual(filter_url('http://domain2.com/app2a/ctr/f2a', domain=('app2a',None), out=True), "/ctr")
+        self.assertEqual(filter_url('http://domain2.com/app2a/c2a/f2a', domain=('app2a',None), out=True), "/")
+        self.assertEqual(filter_url('http://domain2.com/app2a/c2a/fcn', domain=('app2a',None), out=True), "/fcn")
+        
+        self.assertRaises(SyntaxError, filter_url, 'http://domain2.com/app2a/ctr/fcn', domain=('app2b',None), out=True)
+        self.assertRaises(SyntaxError, filter_url, 'http://domain2.com/app2a/ctr/f2a', domain=('app2b',None), out=True)
+        self.assertRaises(SyntaxError, filter_url, 'http://domain2.com/app2a/c2a/f2a', domain=('app2b',None), out=True)
+
+        self.assertEqual(filter_url('http://domain3a.com/'), '/app3/c3a/index')
+        self.assertEqual(filter_url('http://domain3a.com/abc'), '/app3/c3a/abc')
+        self.assertEqual(filter_url('http://domain3a.com/c3b'), '/app3/c3b/index')
+        self.assertEqual(filter_url('http://domain3b.com/abc'), '/app3/c3b/abc')
+
+        self.assertEqual(filter_url('http://domain3a.com/app3/c3a/fcn', domain=('app3','c3a'), out=True), "/fcn")
+        self.assertEqual(filter_url('http://domain3a.com/app3/c3a/fcn', domain=('app3','c3b'), out=True), "/c3a/fcn")
+        
+        self.assertRaises(SyntaxError, filter_url, 'http://domain3a.com/app3/c3a/fcn', domain=('app1',None), out=True)
+
+        self.assertEqual(filter_url('http://domain4a.com/abc'), '/app4/c4/abc')
+        self.assertEqual(filter_url('https://domain4a.com/app4/c4/fcn', domain=('app4',None), out=True), "/fcn")
+
+        self.assertEqual(filter_url('http://domain4a.com'), '/app4/c4/f4a')
+        self.assertEqual(filter_url('http://domain4b.com'), '/app4/c4/f4b')
+
+        self.assertEqual(filter_url('http://localhost/abc'), '/app5/c5/abc')
+        self.assertEqual(filter_url('http:///abc'), '/app5/c5/abc') # test null host => localhost
+        self.assertEqual(filter_url('https://localhost/app5/c5/fcn', domain=('app5',None), out=True), "/fcn")
+
+        self.assertEqual(filter_url('http://domain6.com'), '/app6/c6/f6')
+        self.assertEqual(filter_url('https://domain6.com'), '/app6s/c6s/f6s')
+
+        self.assertRaises(SyntaxError, filter_url, 'http://domain2.com/app3/c3a/f3', domain=('app2b',None), out=True)
+        self.assertRaises(SyntaxError, filter_url, 'http://domain1.com/app1/c1/f1', domain=('app2b',None), out=True)
+        try:
+            # 2.7+ only
+            self.assertRaisesRegexp(SyntaxError, 'cross-domain conflict', filter_url, 'http://domain1.com/app1/c1/f1', domain=('app2b',None), out=True)
+        except AttributeError:
+            pass
+        self.assertEqual(filter_url('http://domain1.com/app1/c1/f1', domain=('app2b',None), host='domain2.com', out=True), "/app1")
+
     def test_router_raise(self):
         '''
         Test URLs that raise exceptions
@@ -533,7 +631,7 @@ class TestRouter(unittest.TestCase):
         self.assertEqual(filter_url('https://domain.com/init/ctr/index', out=True), "/ctr")
         self.assertEqual(filter_url('http://domain.com/init/default/fcn?query', out=True), "/fcn?query")
         self.assertEqual(filter_url('http://domain.com/init/default/fcn#anchor', out=True), "/fcn#anchor")
-        self.assertEqual(filter_url('http://domain.com/init/default/fcn?query#anchor', out=True), 
+        self.assertEqual(filter_url('http://domain.com/init/default/fcn?query#anchor', out=True),
             "/fcn?query#anchor")
 
         router_out['BASE']['map_static'] = True
@@ -541,7 +639,7 @@ class TestRouter(unittest.TestCase):
         self.assertEqual(filter_url('https://domain.com/init/static/file', out=True), "/static/file")
         self.assertEqual(filter_url('https://domain.com/init/static/index', out=True), "/static/index")
 
-        router_out['init']['map_static'] = False
+        router_out['init']['map_static'] = None
         load(rdict=router_out)
         self.assertEqual(filter_url('https://domain.com/init/static/file', out=True), "/init/static/file")
         self.assertEqual(filter_url('https://domain.com/init/static/index', out=True), "/init/static/index")
@@ -575,11 +673,11 @@ class TestRouter(unittest.TestCase):
             ),
         )
         load(rdict=router_functions)
-        
+
         # outbound
         self.assertEqual(str(URL(a='init', c='default', f='f', args=['arg1'])), "/init/f/arg1")
         self.assertEqual(str(URL(a='init', c='default', f='index', args=['arg1'])), "/init/index/arg1")
-        
+
         self.assertEqual(str(URL(a='app', c='default', f='index', args=['arg1'])), "/arg1")
         self.assertEqual(str(URL(a='app', c='default', f='user', args=['arg1'])), "/user/arg1")
         self.assertEqual(str(URL(a='app', c='default', f='user', args=['index'])), "/user/index")
@@ -611,6 +709,45 @@ class TestRouter(unittest.TestCase):
         self.assertEqual(filter_url('http://d.com/app2/ctr/index/arg'), "/app2/ctr/index ['arg']")
         self.assertEqual(filter_url('http://d.com/app2/ctr/arg'), "/app2/ctr/arg")
 
+    def test_router_functions2(self):
+        '''
+        Test more functions=[something]
+        '''
+        router_functions = dict(
+           BASE = dict(
+               default_application = 'init',
+                   applications = 'INIT',
+           ),
+           init = dict(
+               #default_controller = 'default',
+                   controllers = ['default', 'ctr'],
+               #default_function = 'index',
+                   functions = ['index','user','register','basicRegister',
+                       'download','call','data','error']
+           ),
+        )
+
+        load(rdict=router_functions)
+
+        # outbound
+        self.assertEqual(str(URL(a='init', c='default', f='index', args=['arg1'])), "/arg1")
+        self.assertEqual(str(URL(a='init', c='default', f='user', args=['arg1'])), "/user/arg1")
+        self.assertEqual(str(URL(a='init', c='default', f='user', args=['index'])), "/user/index")
+        self.assertEqual(str(URL(a='init', c='default', f='index', args=['index'])), "/index/index")
+        self.assertEqual(str(URL(a='init', c='default', f='index', args=['init'])), "/init")
+        self.assertEqual(str(URL(a='init', c='default', f='index', args=['ctr'])), "/index/ctr")
+        self.assertEqual(str(URL(a='init', c='ctr', f='index', args=['arg'])), "/ctr/index/arg")
+        self.assertEqual(str(URL(a='init', c='ctr', f='ctrf1', args=['arg'])), "/ctr/ctrf1/arg")
+        self.assertEqual(str(URL(a='init', c='ctr', f='ctrf1', args=['ctrf2'])), "/ctr/ctrf1/ctrf2")
+        self.assertEqual(str(URL(a='init', c='default', f='register')), "/register")
+
+        # inbound
+        self.assertEqual(filter_url('http://d.com/arg'), "/init/default/index ['arg']")
+        self.assertEqual(filter_url('http://d.com/user'), "/init/default/user")
+        self.assertEqual(filter_url('http://d.com/user/arg'), "/init/default/user ['arg']")
+        self.assertEqual(filter_url('http://d.com/ctr'), "/init/ctr/index")
+        self.assertEqual(filter_url('http://d.com/ctr/ctrf1/arg'), "/init/ctr/ctrf1 ['arg']")
+
     def test_router_hyphen(self):
         '''
         Test hyphen conversion
@@ -633,18 +770,18 @@ class TestRouter(unittest.TestCase):
         )
         load(rdict=router_hyphen)
         self.assertEqual(filter_url('http://domain.com/init/default/fcn_1', out=True), "/fcn_1")
-        self.assertEqual(filter_url('http://domain.com/static/filename-with_underscore'), 
+        self.assertEqual(filter_url('http://domain.com/static/filename-with_underscore'),
             "%s/applications/init/static/filename-with_underscore" % root)
-        self.assertEqual(filter_url('http://domain.com/init/static/filename-with_underscore', out=True), 
+        self.assertEqual(filter_url('http://domain.com/init/static/filename-with_underscore', out=True),
             "/init/static/filename-with_underscore")
 
-        self.assertEqual(filter_url('http://domain.com/app2/fcn_1'), 
+        self.assertEqual(filter_url('http://domain.com/app2/fcn_1'),
             "/app2/default/fcn_1")
-        self.assertEqual(filter_url('http://domain.com/app2/ctr/fcn_1', domain=('app2',None), out=True), 
+        self.assertEqual(filter_url('http://domain.com/app2/ctr/fcn_1', domain=('app2',None), out=True),
             "/ctr/fcn_1")
-        self.assertEqual(filter_url('http://domain.com/app2/static/filename-with_underscore', domain=('app2',None), out=True), 
+        self.assertEqual(filter_url('http://domain.com/app2/static/filename-with_underscore', domain=('app2',None), out=True),
             "/app2/static/filename-with_underscore")
-        self.assertEqual(filter_url('http://domain.com/app2/static/filename-with_underscore'), 
+        self.assertEqual(filter_url('http://domain.com/app2/static/filename-with_underscore'),
             "%s/applications/app2/static/filename-with_underscore" % root)
 
         self.assertEqual(str(URL(a='init', c='default', f='a_b')), "/a_b")
@@ -691,7 +828,7 @@ class TestRouter(unittest.TestCase):
         self.assertEqual(filter_url('https://domain.com/admin/static/file', lang='it', out=True), "/admin/it/static/file")
         self.assertEqual(filter_url('https://domain.com/admin/static/file', lang='it-it', out=True), "/admin/it-it/static/file")
         self.assertEqual(filter_url('https://domain.com/welcome/ctr/fcn', lang='it', out=True), "/welcome/ctr/fcn")
-        self.assertEqual(filter_url('https://domain.com/welcome/ctr/fcn', lang='es', out=True), "/welcome/ctr/fcn")        
+        self.assertEqual(filter_url('https://domain.com/welcome/ctr/fcn', lang='es', out=True), "/welcome/ctr/fcn")
 
         router_lang['admin']['map_static'] = True
         load(rdict=router_lang)
@@ -702,7 +839,29 @@ class TestRouter(unittest.TestCase):
         self.assertEqual(filter_url('https://domain.com/admin/static/file', lang='it', out=True), "/it/static/file")
         self.assertEqual(filter_url('https://domain.com/admin/static/file', lang='it-it', out=True), "/it-it/static/file")
         self.assertEqual(filter_url('https://domain.com/welcome/ctr/fcn', lang='it', out=True), "/welcome/ctr/fcn")
-        self.assertEqual(filter_url('https://domain.com/welcome/ctr/fcn', lang='es', out=True), "/welcome/ctr/fcn")        
+        self.assertEqual(filter_url('https://domain.com/welcome/ctr/fcn', lang='es', out=True), "/welcome/ctr/fcn")
+
+        router_lang['admin']['map_static'] = False
+        router_lang['examples']['map_static'] = False
+        load(rdict=router_lang)
+        self.assertEqual(filter_url('https://domain.com/admin/ctr/fcn', lang='en', out=True), "/ctr/fcn")
+        self.assertEqual(filter_url('https://domain.com/admin/ctr/fcn', lang='it', out=True), "/it/ctr/fcn")
+        self.assertEqual(filter_url('https://domain.com/admin/ctr/fcn', lang='it-it', out=True), "/it-it/ctr/fcn")
+        self.assertEqual(filter_url('https://domain.com/admin/static/file', lang='en', out=True), "/admin/static/en/file")
+        self.assertEqual(filter_url('https://domain.com/admin/static/file', lang='it', out=True), "/admin/static/it/file")
+        self.assertEqual(filter_url('https://domain.com/admin/static/file', lang='it-it', out=True), "/admin/static/it-it/file")
+        self.assertEqual(filter_url('https://domain.com/welcome/ctr/fcn', lang='it', out=True), "/welcome/ctr/fcn")
+        self.assertEqual(filter_url('https://domain.com/welcome/ctr/fcn', lang='es', out=True), "/welcome/ctr/fcn")
+        self.assertEqual(filter_url('http://domain.com/static/file'), "%s/applications/admin/static/file" % root)
+        self.assertEqual(filter_url('http://domain.com/en/static/file'), "%s/applications/admin/static/file" % root)
+        self.assertEqual(filter_url('http://domain.com/examples/en/static/file'), "%s/applications/examples/static/en/file" % root)
+        self.assertEqual(filter_url('http://domain.com/examples/static/file'), "%s/applications/examples/static/en/file" % root)
+        self.assertEqual(filter_url('http://domain.com/examples/it/static/file'), "%s/applications/examples/static/it/file" % root)
+        self.assertEqual(filter_url('http://domain.com/examples/it-it/static/file'), "%s/applications/examples/static/file" % root)
+
+        self.assertEqual(filter_url('http://domain.com/examples/static/en/file'), "%s/applications/examples/static/en/file" % root)
+        self.assertEqual(filter_url('http://domain.com/examples/static/it/file'), "%s/applications/examples/static/it/file" % root)
+        self.assertEqual(filter_url('http://domain.com/examples/static/it-it/file'), "%s/applications/examples/static/it-it/file" % root)
 
     def test_router_get_effective(self):
         '''
@@ -755,26 +914,69 @@ class TestRouter(unittest.TestCase):
         self.assertEqual(filter_err(399), 399)
         self.assertEqual(filter_err(400), 400)
 
+    def test_router_static_path(self):
+        '''
+        Test validation of static paths
+        Stock pattern: file_match = r'([-+=@$%\w]+[./]?)+$'
+
+        '''
+        load(rdict=dict())
+        self.assertEqual(filter_url('http://domain.com/welcome/static/path/to/static'), "%s/applications/welcome/static/path/to/static" % root)
+        self.assertRaises(HTTP, filter_url, 'http://domain.com/welcome/static/bad/path/to/st~tic')
+        self.assertEqual(filter_url('http://domain.com/welcome/static/path/to--/static'), "%s/applications/welcome/static/path/to--/static" % root)
+        self.assertEqual(filter_url('http://domain.com/welcome/static/path/==to--/static'), "%s/applications/welcome/static/path/==to--/static" % root)
+        self.assertEqual(filter_url('http://domain.com/welcome/static/path/-+=@$%/static'), "%s/applications/welcome/static/path/-+=@$%%/static" % root)
+        self.assertRaises(HTTP, filter_url, 'http://domain.com/welcome/static/bad/path/to/.static')
+        self.assertRaises(HTTP, filter_url, 'http://domain.com/welcome/static/bad/path/to/s..tatic')
+        self.assertRaises(HTTP, filter_url, 'http://domain.com/welcome/static/bad/path/to//static')
+        self.assertRaises(HTTP, filter_url, 'http://domain.com/welcome/static/bad/path/to/#static')
+
+        router_static = dict(
+            BASE = dict(
+                file_match = r'([-+=@$%#\w]+[./]?)+$',   # legal static path
+            ),
+        )
+        load(rdict=router_static)
+        self.assertEqual(filter_url('http://domain.com/welcome/static/path/to/#static'), "%s/applications/welcome/static/path/to/#static" % root)
+
+        router_static = dict(
+            BASE = dict(
+                file_match = r'[-+=@$%#.\w]+$',   # legal static path element
+            ),
+        )
+        load(rdict=router_static)
+        self.assertEqual(filter_url('http://domain.com/welcome/static/path/to/static'), "%s/applications/welcome/static/path/to/static" % root)
+        self.assertRaises(HTTP, filter_url, 'http://domain.com/welcome/static/bad/path/to/st~tic')
+        self.assertEqual(filter_url('http://domain.com/welcome/static/path/to--/static'), "%s/applications/welcome/static/path/to--/static" % root)
+        self.assertEqual(filter_url('http://domain.com/welcome/static/path/==to--/static'), "%s/applications/welcome/static/path/==to--/static" % root)
+        self.assertEqual(filter_url('http://domain.com/welcome/static/path/-+=@$%/static'), "%s/applications/welcome/static/path/-+=@$%%/static" % root)
+        self.assertRaises(HTTP, filter_url, 'http://domain.com/welcome/static/bad/path/to//static')
+        self.assertEqual(filter_url('http://domain.com/welcome/static/path/to/#static'), "%s/applications/welcome/static/path/to/#static" % root)
+        self.assertRaises(HTTP, filter_url, 'http://domain.com/welcome/static/bad/path/./static')
+        self.assertRaises(HTTP, filter_url, 'http://domain.com/welcome/static/bad/path/../static')
+        self.assertEqual(filter_url('http://domain.com/welcome/static/path/.../static'), "%s/applications/welcome/static/path/.../static" % root)
+        self.assertEqual(filter_url('http://domain.com/welcome/static/path/to/.static'), "%s/applications/welcome/static/path/to/.static" % root)
+
     def test_router_args(self):
         '''
         Test URL args parsing/generation
         '''
         load(rdict=dict())
-        self.assertEqual(filter_url('http://domain.com/init/default/f/arg1'), 
+        self.assertEqual(filter_url('http://domain.com/init/default/f/arg1'),
             "/init/default/f ['arg1']")
-        self.assertEqual(filter_url('http://domain.com/init/default/f/arg1/'), 
+        self.assertEqual(filter_url('http://domain.com/init/default/f/arg1/'),
             "/init/default/f ['arg1']")
-        self.assertEqual(filter_url('http://domain.com/init/default/f/arg1//'), 
+        self.assertEqual(filter_url('http://domain.com/init/default/f/arg1//'),
             "/init/default/f ['arg1', '']")
-        self.assertEqual(filter_url('http://domain.com/init/default/f//arg1'), 
+        self.assertEqual(filter_url('http://domain.com/init/default/f//arg1'),
             "/init/default/f ['', 'arg1']")
-        self.assertEqual(filter_url('http://domain.com/init/default/f/arg1/arg2'), 
+        self.assertEqual(filter_url('http://domain.com/init/default/f/arg1/arg2'),
             "/init/default/f ['arg1', 'arg2']")
-        self.assertEqual(filter_url('http://domain.com/init/default/f/arg1//arg2'), 
+        self.assertEqual(filter_url('http://domain.com/init/default/f/arg1//arg2'),
             "/init/default/f ['arg1', '', 'arg2']")
-        self.assertEqual(filter_url('http://domain.com/init/default/f/arg1//arg3/'), 
+        self.assertEqual(filter_url('http://domain.com/init/default/f/arg1//arg3/'),
             "/init/default/f ['arg1', '', 'arg3']")
-        self.assertEqual(filter_url('http://domain.com/init/default/f/arg1//arg3//'), 
+        self.assertEqual(filter_url('http://domain.com/init/default/f/arg1//arg3//'),
             "/init/default/f ['arg1', '', 'arg3', '']")
 
         self.assertEqual(filter_url('http://domain.com/init/default/f', out=True), "/f")
@@ -798,12 +1000,12 @@ class TestRouter(unittest.TestCase):
         load(rdict=dict())
         self.assertEqual(str(URL(a='a', c='c', f='f', anchor='anchor')), "/a/c/f#anchor")
         args = ['a1', 'a2']
-        self.assertEqual(str(URL(a='a', c='c', f='f', args=args, anchor='anchor')), 
+        self.assertEqual(str(URL(a='a', c='c', f='f', args=args, anchor='anchor')),
             "/a/c/f/a1/a2#anchor")
         vars = dict(v1=1, v2=2)
-        self.assertEqual(str(URL(a='a', c='c', f='f', vars=vars, anchor='anchor')), 
+        self.assertEqual(str(URL(a='a', c='c', f='f', vars=vars, anchor='anchor')),
             "/a/c/f?v1=1&v2=2#anchor")
-        self.assertEqual(str(URL(a='a', c='c', f='f', args=args, vars=vars, anchor='anchor')), 
+        self.assertEqual(str(URL(a='a', c='c', f='f', args=args, vars=vars, anchor='anchor')),
             "/a/c/f/a1/a2?v1=1&v2=2#anchor")
         self.assertEqual(str(URL(a='init', c='default', f='index')),
             "/")
@@ -833,11 +1035,11 @@ class TestRouter(unittest.TestCase):
             ),
         )
         load(rdict=router_path_prefix)
-        self.assertEqual(str(URL(a='a1', c='c1a', f='f')), 
+        self.assertEqual(str(URL(a='a1', c='c1a', f='f')),
             "/path/to/apps/c1a/f")
-        self.assertEqual(str(URL(a='a2', c='c', f='f')), 
+        self.assertEqual(str(URL(a='a2', c='c', f='f')),
             "/path/to/apps/a2/c/f")
-        self.assertEqual(str(URL(a='a2', c='c2', f='f')), 
+        self.assertEqual(str(URL(a='a2', c='c2', f='f')),
             "/path/to/apps/a2/c2/f")
         self.assertEqual(filter_url('http://domain.com/a1/'), "/a1/default/index")
         self.assertEqual(filter_url('http://domain.com/path/to/apps/a1/'), "/a1/default/index")
@@ -853,35 +1055,35 @@ class TestRouter(unittest.TestCase):
         r.env.http_host = 'domain.com'
         r.env.wsgi_url_scheme = 'httpx' # distinguish incoming scheme
         self.assertEqual(str(URL(r=r, a='a', c='c', f='f')), "/a/c/f")
-        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', host=True)), 
+        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', host=True)),
             "httpx://domain.com/a/c/f")
-        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', host='host.com')), 
+        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', host='host.com')),
             "httpx://host.com/a/c/f")
-        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', scheme=True)), 
+        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', scheme=True)),
             "httpx://domain.com/a/c/f")
-        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', scheme=False)), 
+        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', scheme=False)),
             "/a/c/f")
-        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', scheme='https')), 
+        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', scheme='https')),
             "https://domain.com/a/c/f")
-        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', scheme='wss')), 
+        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', scheme='wss')),
             "wss://domain.com/a/c/f")
-        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', scheme=True, host=True)), 
+        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', scheme=True, host=True)),
             "httpx://domain.com/a/c/f")
-        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', scheme='https', host=True)), 
+        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', scheme='https', host=True)),
             "https://domain.com/a/c/f")
-        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', scheme=False, host=True)), 
+        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', scheme=False, host=True)),
             "httpx://domain.com/a/c/f")
-        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', scheme=True, host='host.com')), 
+        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', scheme=True, host='host.com')),
             "httpx://host.com/a/c/f")
-        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', scheme=False, host='host.com')), 
+        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', scheme=False, host='host.com')),
             "httpx://host.com/a/c/f")
-        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', port=1234)), 
+        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', port=1234)),
             "httpx://domain.com:1234/a/c/f")
-        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', scheme=True, port=1234)), 
+        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', scheme=True, port=1234)),
             "httpx://domain.com:1234/a/c/f")
-        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', host='host.com', port=1234)), 
+        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', host='host.com', port=1234)),
             "httpx://host.com:1234/a/c/f")
-        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', scheme='wss', host='host.com', port=1234)), 
+        self.assertEqual(str(URL(r=r, a='a', c='c', f='f', scheme='wss', host='host.com', port=1234)),
             "wss://host.com:1234/a/c/f")
 
     def test_request_uri(self):
@@ -890,15 +1092,15 @@ class TestRouter(unittest.TestCase):
         '''
         load(rdict=dict())
 
-        self.assertEqual(filter_url('http://domain.com/abc', env=True).request_uri, 
+        self.assertEqual(filter_url('http://domain.com/abc', env=True).request_uri,
             '/init/default/abc')
-        self.assertEqual(filter_url('http://domain.com/abc?def', env=True).request_uri, 
+        self.assertEqual(filter_url('http://domain.com/abc?def', env=True).request_uri,
             '/init/default/abc?def')
-        self.assertEqual(filter_url('http://domain.com/index/abc', env=True).request_uri, 
+        self.assertEqual(filter_url('http://domain.com/index/abc', env=True).request_uri,
             "/init/default/index/abc")
-        self.assertEqual(filter_url('http://domain.com/abc/def', env=True).request_uri, 
+        self.assertEqual(filter_url('http://domain.com/abc/def', env=True).request_uri,
             "/init/default/abc/def")
-        self.assertEqual(filter_url('http://domain.com/index/a%20bc', env=True).request_uri, 
+        self.assertEqual(filter_url('http://domain.com/index/a%20bc', env=True).request_uri,
             "/init/default/index/a%20bc")
 
     def test_request_collide(self):
@@ -917,7 +1119,7 @@ class TestRouter(unittest.TestCase):
             ),
         )
         load(rdict=router_collide)
-        
+
         # basic inbound
         self.assertEqual(filter_url('http://ex.domain.com'), '/examples/default/exdef')
         self.assertEqual(filter_url('http://ad.domain.com'), '/admin/default/index')
@@ -948,3 +1150,4 @@ if __name__ == '__main__':
     setUpModule()       # pre-2.7
     unittest.main()
     tearDownModule()
+
